@@ -1,5 +1,5 @@
 /* Test of conversion to UTF-32 from legacy encodings.
-   Copyright (C) 2007-2016 Free Software Foundation, Inc.
+   Copyright (C) 2007-2018 Free Software Foundation, Inc.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -12,7 +12,7 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
+   along with this program.  If not, see <https://www.gnu.org/licenses/>.  */
 
 /* Written by Bruno Haible <bruno@clisp.org>, 2007.  */
 
@@ -25,6 +25,7 @@
 
 #include "unistr.h"
 #include "macros.h"
+extern int iconv_supports_encoding (const char *encoding);
 
 /* Magic number for detecting bounds violations.  */
 #define MAGIC 0x1983EFF1
@@ -117,98 +118,101 @@ main ()
 
   /* autodetect_jp is only supported when iconv() support ISO-2022-JP-2.  */
 # if defined _LIBICONV_VERSION || !(defined _AIX || defined __sgi || defined __hpux || defined __osf__ || defined __sun)
-  /* Test conversions from autodetect_jp to UTF-16.  */
-  for (h = 0; h < SIZEOF (handlers); h++)
+  if (iconv_supports_encoding ("ISO-2022-JP-2"))
     {
-      enum iconv_ilseq_handler handler = handlers[h];
-      static const char input[] = "\244\263\244\363\244\313\244\301\244\317"; /* こんにちは in EUC-JP */
-      static const uint32_t expected[] = /* こんにちは */
+      /* Test conversions from autodetect_jp to UTF-16.  */
+      for (h = 0; h < SIZEOF (handlers); h++)
         {
-          0x3053, 0x3093, 0x306B, 0x3061, 0x306F
-        };
-      for (o = 0; o < 2; o++)
-        {
-          size_t *offsets = (o ? new_offsets (strlen (input)) : NULL);
-          size_t length;
-          uint32_t *result = u32_conv_from_encoding ("autodetect_jp", handler,
-                                                     input, strlen (input),
-                                                     offsets,
-                                                     NULL, &length);
-          ASSERT (result != NULL);
-          ASSERT (length == SIZEOF (expected));
-          ASSERT (u32_cmp (result, expected, SIZEOF (expected)) == 0);
-          if (o)
+          enum iconv_ilseq_handler handler = handlers[h];
+          static const char input[] = "\244\263\244\363\244\313\244\301\244\317"; /* こんにちは in EUC-JP */
+          static const uint32_t expected[] = /* こんにちは */
             {
-              for (i = 0; i < 10; i++)
-                ASSERT (offsets[i] == ((i % 2) == 0 ? i / 2 : (size_t)(-1)));
-              ASSERT (offsets[10] == MAGIC);
-              free (offsets);
+              0x3053, 0x3093, 0x306B, 0x3061, 0x306F
+            };
+          for (o = 0; o < 2; o++)
+            {
+              size_t *offsets = (o ? new_offsets (strlen (input)) : NULL);
+              size_t length;
+              uint32_t *result = u32_conv_from_encoding ("autodetect_jp", handler,
+                                                         input, strlen (input),
+                                                         offsets,
+                                                         NULL, &length);
+              ASSERT (result != NULL);
+              ASSERT (length == SIZEOF (expected));
+              ASSERT (u32_cmp (result, expected, SIZEOF (expected)) == 0);
+              if (o)
+                {
+                  for (i = 0; i < 10; i++)
+                    ASSERT (offsets[i] == ((i % 2) == 0 ? i / 2 : (size_t)(-1)));
+                  ASSERT (offsets[10] == MAGIC);
+                  free (offsets);
+                }
+              free (result);
             }
-          free (result);
         }
-    }
-  for (h = 0; h < SIZEOF (handlers); h++)
-    {
-      enum iconv_ilseq_handler handler = handlers[h];
-      static const char input[] = "\202\261\202\361\202\311\202\277\202\315"; /* こんにちは in Shift_JIS */
-      static const uint32_t expected[] = /* こんにちは */
+      for (h = 0; h < SIZEOF (handlers); h++)
         {
-          0x3053, 0x3093, 0x306B, 0x3061, 0x306F
-        };
-      for (o = 0; o < 2; o++)
-        {
-          size_t *offsets = (o ? new_offsets (strlen (input)) : NULL);
-          size_t length;
-          uint32_t *result = u32_conv_from_encoding ("autodetect_jp", handler,
-                                                     input, strlen (input),
-                                                     offsets,
-                                                     NULL, &length);
-          ASSERT (result != NULL);
-          ASSERT (length == SIZEOF (expected));
-          ASSERT (u32_cmp (result, expected, SIZEOF (expected)) == 0);
-          if (o)
+          enum iconv_ilseq_handler handler = handlers[h];
+          static const char input[] = "\202\261\202\361\202\311\202\277\202\315"; /* こんにちは in Shift_JIS */
+          static const uint32_t expected[] = /* こんにちは */
             {
-              for (i = 0; i < 10; i++)
-                ASSERT (offsets[i] == ((i % 2) == 0 ? i / 2 : (size_t)(-1)));
-              ASSERT (offsets[10] == MAGIC);
-              free (offsets);
+              0x3053, 0x3093, 0x306B, 0x3061, 0x306F
+            };
+          for (o = 0; o < 2; o++)
+            {
+              size_t *offsets = (o ? new_offsets (strlen (input)) : NULL);
+              size_t length;
+              uint32_t *result = u32_conv_from_encoding ("autodetect_jp", handler,
+                                                         input, strlen (input),
+                                                         offsets,
+                                                         NULL, &length);
+              ASSERT (result != NULL);
+              ASSERT (length == SIZEOF (expected));
+              ASSERT (u32_cmp (result, expected, SIZEOF (expected)) == 0);
+              if (o)
+                {
+                  for (i = 0; i < 10; i++)
+                    ASSERT (offsets[i] == ((i % 2) == 0 ? i / 2 : (size_t)(-1)));
+                  ASSERT (offsets[10] == MAGIC);
+                  free (offsets);
+                }
+              free (result);
             }
-          free (result);
         }
-    }
-  for (h = 0; h < SIZEOF (handlers); h++)
-    {
-      enum iconv_ilseq_handler handler = handlers[h];
-      static const char input[] = "\033$B$3$s$K$A$O\033(B"; /* こんにちは in ISO-2022-JP-2 */
-      static const uint32_t expected[] = /* こんにちは */
+      for (h = 0; h < SIZEOF (handlers); h++)
         {
-          0x3053, 0x3093, 0x306B, 0x3061, 0x306F
-        };
-      for (o = 0; o < 2; o++)
-        {
-          size_t *offsets = (o ? new_offsets (strlen (input)) : NULL);
-          size_t length;
-          uint32_t *result = u32_conv_from_encoding ("autodetect_jp", handler,
-                                                     input, strlen (input),
-                                                     offsets,
-                                                     NULL, &length);
-          ASSERT (result != NULL);
-          ASSERT (length == SIZEOF (expected));
-          ASSERT (u32_cmp (result, expected, SIZEOF (expected)) == 0);
-          if (o)
+          enum iconv_ilseq_handler handler = handlers[h];
+          static const char input[] = "\033$B$3$s$K$A$O\033(B"; /* こんにちは in ISO-2022-JP-2 */
+          static const uint32_t expected[] = /* こんにちは */
             {
-              for (i = 0; i < 16; i++)
-                ASSERT (offsets[i] == (i == 0 ? 0 :
-                                       i == 5 ? 1 :
-                                       i == 7 ? 2 :
-                                       i == 9 ? 3 :
-                                       i == 11 ? 4 :
-                                       i == 13 ? 5 :
-                                       (size_t)(-1)));
-              ASSERT (offsets[16] == MAGIC);
-              free (offsets);
+              0x3053, 0x3093, 0x306B, 0x3061, 0x306F
+            };
+          for (o = 0; o < 2; o++)
+            {
+              size_t *offsets = (o ? new_offsets (strlen (input)) : NULL);
+              size_t length;
+              uint32_t *result = u32_conv_from_encoding ("autodetect_jp", handler,
+                                                         input, strlen (input),
+                                                         offsets,
+                                                         NULL, &length);
+              ASSERT (result != NULL);
+              ASSERT (length == SIZEOF (expected));
+              ASSERT (u32_cmp (result, expected, SIZEOF (expected)) == 0);
+              if (o)
+                {
+                  for (i = 0; i < 16; i++)
+                    ASSERT (offsets[i] == (i == 0 ? 0 :
+                                           i == 5 ? 1 :
+                                           i == 7 ? 2 :
+                                           i == 9 ? 3 :
+                                           i == 11 ? 4 :
+                                           i == 13 ? 5 :
+                                           (size_t)(-1)));
+                  ASSERT (offsets[16] == MAGIC);
+                  free (offsets);
+                }
+              free (result);
             }
-          free (result);
         }
     }
 # endif
